

#include <stdint.h>


using namespace std;


#define  RCU_APB2EN  *((volatile uint32_t *) 0x40021018)
#define  GPIOC_CTL1  *((volatile uint32_t *) 0x40011004)
#define  GPIOC_OCTL  *((volatile uint32_t *) 0x4001100C)
#define  MTIME       *((volatile uint64_t *) 0xD1000000)
#define  MTIMECMP    *((volatile uint64_t *) 0xD1000008)


#define  MTIME_INTERRUPT_PERIOD  12000000    // 24 MHz, so 12000000 generates an interrupt period of exactly half a second


void interruptHandler() __attribute__ ((interrupt, section(".interrupt_handler")));


void interruptHandler() {
	MTIME = 0;
	MTIMECMP = MTIME_INTERRUPT_PERIOD;
	GPIOC_OCTL = GPIOC_OCTL ^ (((uint32_t) 1) << 13);
}


int main() {
	// enable clock on port C
	RCU_APB2EN = RCU_APB2EN | (((uint32_t) 1) << 4);
	// PC13 pin is output, low speed, push-pull
	GPIOC_CTL1 = 0x44244444;
	// basic (non vectored) interrupt handler (to force non vectored, set 0 to lower two bits of mtvec, so force 4 byte aligned on linker script for interrupt handler)
	asm volatile (
		"csrw mtvec, %[reg]" : : [reg] "r" ((uint32_t) interruptHandler)
	);
	// machine interrupt enable
	asm volatile (
		"csrw mie, %[reg]" : : [reg] "r" ((uint32_t) 0x80)
	);
	asm volatile (
		"csrsi mstatus, 8"
	);
	// configure interrupt period
	MTIME = 0;
	MTIMECMP = MTIME_INTERRUPT_PERIOD;
	// sleep
	while (true)
		asm volatile ("wfi");
}
